(
	/*
	Matlib Refresher
	Changes some old defaults in vrayMtls and some maps to modern ones.
	Aims to process .mat and .max files in batch.
	
	v0.05 = strenghtens file holding/fetching, adds vrayBitmap conversion from vrutils.ms
	*/
	
	global mtlLibProcRoll
	try (destroyDialog mtlLibProcRoll) catch ()
	
	local msgString = ""
	local theMatLibRoot = ""
	local theMatlibs = #()
	local theMaxFiles = #()
	local hasLoaded = false
	local OpMode = 1 --1 is open scene, 2 is disk-based files
	local theMats = #()
	local appendage = ""
	local wasHeld=false
	
	local doGGX = true --switch BRDF to GGX
	local doGF = true --Turn on Glossy Fresnel (makes sense only for GGX)
	local doEC = true --Turn on Energy Conservation (makes sense only for GGX)
	local doCO = true --Sets cutoff values to 0.001 (old default was 0.01)
	local doCL = true --Uses the embedded max scene converter to remove legacy mentalray custom attributes
	local doBL = true --uses vrutils.ms's functions to replace bitmap loaders for V-Ray ones in the open scene.
	local doBlur = true --regathers a scene's vraybitmaps and sets blur to 1.0.
	
	local appendMaxVer = true
	local lmaxVer = ""
	local appendCustString = true
	local custString = ""
	local maxExt = ".max"
	
	local overwriteExisting = "always" -- 0-based: always, never, ask

-- 	loadMaterialLibrary @"D:\_VrayMtlLib\Bricks_2023_myCustomString.mat"

	fn getCurrMaxVer =
	(
		lmaxVer = (maxversion())[8] as string
	)
	getCurrMaxVer() --run it right away to fill the variable.
	
	
	--File-fetching functions
	fn getMatlibs =
	(
		theMatLibRoot = getSavePath caption:"Please select a location to scan for material libraries to process"
		if theMatLibRoot != undefined do 
		(
			theMatlibs = getFiles (theMatLibRoot+"\\*.mat") recurse:true
			theMaxFiles = getFiles (theMatLibRoot+"\\*.max") recurse:true
		)
	)

	fn loadMatLib matLib=
	(
		hasLoaded = with quiet on (loadMaterialLibrary matLib) --loading max files as matlibs throws an error but the file loads correctly, hence the quiet mode.
	)

	fn loadMaxF maxFile = 
	(
		hasLoaded = loadMaxFile maxFile quiet:true useFileUnits:true allowPrompts:false
	)
	
	--UI-related functions
	fn setUI =
	(
		if opMode == 1 then		--open scene only
		(
			mtlLibProcRoll.btn1.text = "Pick a Path to Scan..."
			mtlLibProcRoll.btn1.enabled = mtlLibProcRoll.chk6.enabled = mtlLibProcRoll.chk7.enabled = mtlLibProcRoll.lbl4.enabled = 
			mtlLibProcRoll.lbl3.enabled = mtlLibProcRoll.ddl2.enabled = mtlLibProcRoll.edt1.enabled = mtlLibProcRoll.ddl3.enabled = false
			theMats = for m in sceneMaterials where classof m == vraymtl collect m
			mtlLibProcRoll.lbl1.text=("Found a total of " + theMats.count as string + "\nMaterials in the currently open scene")
		)
		else	--from disk files
		(
			maxExt = mtlLibProcRoll.ddl2.selected
			overwriteExisting = mtlLibProcRoll.ddl3.selected
			mtlLibProcRoll.btn1.enabled = mtlLibProcRoll.chk6.enabled = mtlLibProcRoll.chk7.enabled = mtlLibProcRoll.lbl4.enabled = 
			mtlLibProcRoll.lbl3.enabled = mtlLibProcRoll.ddl2.enabled = mtlLibProcRoll.edt1.enabled = mtlLibProcRoll.ddl3.enabled = true
			if theMatlibs.count + theMaxFiles.count == 0 then
			(
				mtlLibProcRoll.btn1.text = "Pick a Path to Scan..."
				mtlLibProcRoll.lbl1.text=""
			)
			else
			(
				mtlLibProcRoll.btn1.text = theMatLibRoot
				mtlLibProcRoll.lbl1.text=("Found a total of " + theMatlibs.count as string +" Material Libraries\n")
				mtlLibProcRoll.lbl1.text+=("and a total of " + theMaxFiles.count as string +" Max Files\n")
				mtlLibProcRoll.lbl1.text+="in the scanned path(s)"
			)
		)

	)
	
	fn assignMatlibMats inputMatlib =
	(
		for m in inputMatlib where classof m == vraymtl do 
		(
			theMesh = mesh name:("mtlProc_fauxMesh_"+m.name) numverts:1 numfaces:1
			theMesh.material = m
		)
	)

	fn deleteTempMeshes =
	(
		clearSelection()
		for g in geometry where (matchPattern g.name pattern:"mtlProc_fauxMesh_*") do selectmore g
		delete selection
	)
	
	--Process-related functions
	fn cleanOfLegacyAssets =
	(
		if (sceneConverter.GetMissingLegacyAssetsName()).count > 0 do
			(
				SceneConverter.ClearInvalidClassIDs()
				sceneConverter.RemoveMissingLegacyAssetsFromScene (sceneConverter.GetMissingLegacyAssetsName())
			)
	)	

	fn processMaterials doOpenScene =
	(
		
		if doOpenScene then --materials already assigned, collect instances
		(
			theMats = getClassInstances vraymtl --for m in sceneMaterials where classof m == vraymtl collect m
			theMaxBmps = getClassInstances bitmapTex
			theVRMaps = getClassInstances vrayBitmap
		)
		else --assign materials, then collect instances
		(
			assignMatlibMats currentMaterialLibrary
			theMats = getClassInstances vraymtl --for m in sceneMaterials where classof m == vraymtl collect m
			theMaxBmps = getClassInstances bitmapTex
			theVRMaps = getClassInstances vrayBitmap
		)
		
		msgString += "Processing Materials and Maps:\n"
		for m in theMats do
		(
			msgString += m.name + "\n"
			if doGGX do 
			(
				m.brdf_type=4
				if doGF do m.option_glossyFresnel = true			
				if doEC do 
				(
					m.option_gtrEnergyCompensation=true
					m.brdf_newGTRAnisotropy = true
				)
			)
			if doCO do m.option_cutOff = 0.001
			if doCL do cleanOfLegacyAssets()
			if doBL do 
			(
				if VRayBitmap2VRayBitmap != undefined do
				(
					VRayBitmap2VRayBitmap applyOnSceneObjects:true
				)
				if doBlur do
				(
					themaps = getclassinstances vraybitmap
					for m in theMaps do m.coords.blur=1.0
				)
			)
		)
	)

	fn saveMatLib matlib=
	(
		baseFilePath = getFilenamePath matlib
		baseFileName = getFilenameFile matlib
		baseFileType = getfilenameType matlib
		
		if appendMaxVer do baseFileName += "_"+lmaxVer
		if appendCustString do baseFileName += custString
		if baseFileType == ".max" do 
		(
			baseFileType = maxExt --convert .max filename extension to the chose extension between .max and .mat
		)
		matlib = baseFilePath + baseFileName + baseFileType
		msgString += "Saved " + matlib + "\n"
		case overwriteExisting of
		(
			"always":
			(
				if getfilenameType matlib == ".max" then 
				(
					saveMaxFile matlib 
				)
				else 
				(
					saveMaterialLibrary matlib
				)
			)
			"never":
			(
				if not doesfileExist matlib do
				(
					if getfilenameType matlib == ".max" then 
					(
						saveMaxFile matlib 
					)
					else 
					(
						saveMaterialLibrary matlib
					)
				)					
			)
			"ask":
			(
				if doesFileExist matlib do
				(
					Case yesNoCancelBox ("The file " + matlib as string + " already exists. Overwrite?") of
					(
						#yes:
						(
							if getfilenameType matlib == ".max" then 
							(
								saveMaxFile matlib 
							)
							else 
							(
								saveMaterialLibrary matlib
							)
						)
						#no:
						(
						)
						#cancel:
						(
							exit
						)
					)
				)
			)
		)

	)

	--tooltips
	local tooltip_a = "Set the script to work on the scene being currently open in 3D Studio Max, or on .max and .mat files on disk.\n\nRoll the mouse wheel while over the dropdown to change mode without clicking."
	local tooltip_b = "Click to select a path to scan for .mat and .max files to work on."
	local tooltip_c = "Sets the Materials' BRDF to the more modern GTR/GGX (from Blinn/Phong/Ward).\n\nHighlight size and looks will change."
	local tooltip_d = "Activates the Materials' Glossy Fresnel (Needs GTR/GGX to work).\n\nHighlight size and looks will change."
	local tooltip_e = "Activates Energy Conservation (Needs GTR/GGX to work) and the new Anisotropy mode (it *won't* make the highlight Anisotropic.).\n\nHighlight size and looks will change."
	local tooltip_f = "Sets the Materials' cutoff values to 0.001, down from the old default of 0.01.\n\nUseful to avoid sharp cut-offs in reflections."
	local tooltip_g = "Scan for and remove legacy custom attributes (for example Mental Ray ones)."
	local tooltip_h = "Append f.e. \"_2023\" to the filename.\n\nGet \"filename_2023.max\" or \"filename_2023.mat\"."
	local tooltip_i = "Append a custom token to the filename.\n\nThis activates the feature, the token is input in the box on the right."
	local tooltip_j = "Input f.e. \"_customToken\" to get \"filename_customToken.max\" or \"filename_customToken.mat\""
	local tooltip_k = "When processing .max files, open, process and save them as\n.max\n thereby treating them as 3d scenes, or as \n.mat\ntreating them as a material library.\n\nAssuming processing in max 2023, by default the filenames would then be \"maxfile_2023.max\" or \"maxfile_2023.mat\".\n\nRoll the mouse wheel while over the dropdown to change mode without clicking."
	local tooltip_l = "Info Box."
	Local tooltip_m = "Go!"
	local tooltip_n = "Progress measured in filenames (being) processed."
	local tooltip_o = "\"always\" will overwrite any found file without asking.\nIdeal for batch processing of non-original source files.\n\n\"never\" will never overwrite any found file without bothering the user about it.\nSafe for batch processing of original source files as nothing already on disk will ever be overwritten.\n\n\"ask\" will ask the user to confirm each overwrite, allowing them to choose to overwrite, not to overwrite and cancel out entirely.\nThe slowest option, and it's also easy to get caught out. Strongly discouraged."
	local tooltip_p = "Uses standard V-Ray functions (contained in vrutils.ms) to swap the bitmap loaders to V-Ray ones. Notice this will only work for Max files, not matlibs."
	local tooltip_q = "Resets blur to 1.0 for the vrayBitmap loaders. They default to a much sharper filtering mode than Max Bitmap Loaders, and won't need to lower blur as it used to be the case. Notice this will only work for Max files, not matlibs."
	
	rollout mtlLibProcRoll "Material Library Processor v 0.07" width:336 height:(392+8)
	(
		GroupBox 'grp2' "Basic:" pos:[8,8] width:320 height:88 		
		label 'lbl2' "Operation Mode:" pos:[16,28] width:96 height:16 			
		dropdownList 'ddl1' "" pos:[120,24] width:200 height:21 items:#("Work on the currently open scene", "Work on disk-stored files") selection:1 tooltip:tooltip_a
		button 'btn1' "Pick a Path to Scan..." pos:[16,48] width:304 height:40 tooltip:tooltip_b

		GroupBox 'grp3' "Operations:" pos:[8,96] width:320 height:88 		
		checkbox 'chk1' "Set GGX BDRF ON" pos:[16,112] width:120 height:16 checked:true tooltip:tooltip_c
		checkbox 'chk2' "Set Glossy Fresnel ON" pos:[16,128] width:128 height:16 checked:true tooltip:tooltip_d
		checkbox 'chk3' "Set EC+Anisotropy ON" pos:[160,112] width:160 height:16 checked:true tooltip:tooltip_e
		checkbox 'chk4' "Set Cutoffs to 0.001" pos:[160,128] width:120 height:16 checked:true tooltip:tooltip_f
		checkbox 'chk5' "Clean Legacy CAs" pos:[16,144] width:126 height:16 checked:true tooltip:tooltip_g
		checkbox 'chk8' "Swap to vrayBitmap Loaders" pos:[160,144] width:160 height:16 checked:true tooltip:tooltip_p 
		checkbox 'chk9' "Reset Blur to 1.0" pos:[176,160] width:104 height:16 checked:true tooltip:tooltip_q 

		GroupBox 'grp4' "Output:" pos:[8,168+16] width:320 height:80	
		checkbox 'chk6' "Append Max Version to Saved Filename" pos:[16,192+8] width:208 height:16 checked:true tooltip:tooltip_h
		checkbox 'chk7' "Append custom token to Saved Filename:" pos:[16,208+8] width:216 height:16 enabled:true checked:false tooltip:tooltip_i
		edittext 'edt1' "" pos:[240,208+8] width:80 height:16 enabled:true tooltip:tooltip_j	
		label 'lbl3' "Save Max files as:" pos:[16,232+8] width:88 height:16 tooltip:tooltip_k
		dropdownList 'ddl2' "" pos:[112,228+8] width:56 height:21 items:#(".max", ".mat") tooltip:tooltip_k
		label 'lbl4' "Overwrite:" pos:[188,232+8] width:52 height:16 tooltip:tooltip_o
		dropdownList 'ddl3' "" pos:[242,228+8] width:80 height:21 enabled:true items:#("always", "never", "ask")  tooltip:tooltip_o

		GroupBox 'grp1' "InfoBox:" pos:[8,256+8] width:320 height:64 
		label 'lbl1' "" pos:[16,272+8] width:304 height:40 tooltip:tooltip_l
		button 'btn2' "Go!" pos:[8,328+8] width:320 height:32 tooltip:tooltip_m
		progressBar 'pb1' "ProgressBar" pos:[8,368+8] width:320 height:8 tooltip:tooltip_n

		on mtlLibProcRoll open do
		(
			theMatlibs=#()
			theMaxFiles=#()
			hasLoaded = false
			setUI()
		)
		on ddl1 selected sel do
		(
			opMode = sel
			setUI()
		)
		on btn1 pressed do
		(
			getMatlibs()
			setUI()
		)
		on chk1 changed state do
		(
			doGGX = state
			chk2.enabled = chk3.enabled = state
		)
		on chk2 changed state do
		(
			doGF = state
		)
		on chk3 changed state do
		(
			doEC = state
		)
		on chk4 changed state do
		(
			doCO = state
		)
		on chk5 changed state do
		(
			doCL = state
		)
		on chk6 changed state do
		(
			appendMaxVer = state
		)
		on chk7 changed state do
		(
			appendCustString = state
			custString = edt1.text
		)
		on chk8 changed state do 
		(
			doBL = state
			chk9.enabled=state
		)
		on chk9 changed state do
		(
			doBlur = state
		)
		
		on edt1 changed txt do
		(
			custString = txt
		)
		on ddl2 selected sel do
		(
			maxExt = ddl2.selected
			setUI()
		)
		on ddl3 selected sel do
		(
			overwriteExisting = ddl3.selected
			setUI()
		)
		on btn2 pressed do
		(
			wasHeld=false
			st=timestamp()
			if opMode == 1 then
			(
				processMaterials true
			)
			else
			(
				if queryBox "Are you sure you want to process these files with the settings you chose?" then
				(
					holdMaxFile()
					wasHeld=true
					totCount = (theMatLibs.count + theMaxFiles.count) as float
					i=0
					for ml in theMatLibs do
					(
						i+=1
						pb1.value=(i as float/totCount)*100
						loadMatLib ml
						if hasLoaded then
						(
							msgString += "Opened " + ml + " for processing:\n"
							processMaterials false
							saveMatlib ml
						)
						else
						(
							print ("The matlib named " + ml + " didn't load and wasn't processed.")
						)
					)
			
					for ml in theMaxFiles do
					(
						i+=1
						pb1.value=(i as float/totCount)*100
						loadMaxF ml
						if hasLoaded then
						(
							msgString += "Opened " + ml + " for processing:\n"
							processMaterials true
							saveMatlib ml
						)
						else
						(
							print ("The matlib named " + ml + " didn't load and wasn't processed.")
						)
					)
				)
				else
				(
				)
			)
		
			print msgString
			print (((timestamp()-st)/1000.0)as string)
			if wasHeld do fetchMaxFile quiet:true
		)

	)
	
	createDialog mtlLibProcRoll
)
